@@ -5,24 +5,31 @@
 from __future__ import annotations
 
 from abc import ABCMeta
+from configparser import ConfigParser
 
 from livechat.utils.helpers import prepare_payload
+from livechat.utils.structures import RtmResponse
 from livechat.utils.ws_client import WebsocketClient
 
+config = ConfigParser()
+config.read('configs/main.ini')
+stable_version = config.get('api', 'stable')
+api_url = config.get('api', 'url')
+
 
 class CustomerRTM:
     ''' Main class that gets specific client. '''
     @staticmethod
     def get_client(license_id: int = None,
-                   version: str = '3.3',
-                   base_url: str = 'api.livechatinc.com',
+                   version: str = stable_version,
+                   base_url: str = api_url,
                    organization_id: str = None) -> CustomerRTMInterface:
         ''' Returns client for specific Customer RTM version.
 
             Args:
-                license_id (int): License ID. Required to use API v3.3.
-                version (str): API's version. Defaults to `3.3`.
-                base_url (str): API's base url. Defaults to `api.livechatinc.com`.
+                license_id (int): License ID. Required to use for API version <= 3.3.
+                version (str): API's version. Defaults to the stable version of API.
+                base_url (str): API's base url. Defaults to API's production URL.
                 organization_id (str): Organization ID, replaced license ID in v3.4.
 
             Returns:
@@ -31,7 +38,11 @@ class CustomerRTM:
             Raises:
                 ValueError: If the specified version does not exist.
         '''
-        client = {'3.3': CustomerRTM33, '3.4': CustomerRTM34}.get(version)
+        client = {
+            '3.3': CustomerRTM33,
+            '3.4': CustomerRTM34,
+            '3.5': CustomerRTM35,
+        }.get(version)
         client_kwargs = {
             '3.3': {
                 'license_id': license_id,
@@ -42,7 +53,12 @@ class CustomerRTM:
                 'organization_id': organization_id,
                 'version': version,
                 'url': base_url
-            }
+            },
+            '3.5': {
+                'organization_id': organization_id,
+                'version': version,
+                'url': base_url
+            },
         }.get(version)
         if client:
             return client(**client_kwargs)
@@ -71,7 +87,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
         ''' Closes WebSocket connection. '''
         self.ws.close()
 
-    def login(self, token: str = None, payload: dict = None) -> dict:
+    def login(self, token: str = None, payload: dict = None) -> RtmResponse:
         ''' Logs in customer.
 
             Args:
@@ -80,7 +96,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -89,7 +106,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
     def list_group_statuses(self,
                             all: bool = None,
                             group_ids: list = None,
-                            payload: dict = None) -> dict:
+                            payload: dict = None) -> RtmResponse:
         ''' Lists statuses of groups.
 
             Args:
@@ -99,7 +116,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -112,7 +130,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                    chat: dict = None,
                    active: bool = None,
                    continuous: bool = None,
-                   payload: dict = None) -> dict:
+                   payload: dict = None) -> RtmResponse:
         ''' Starts a chat.
 
             Args:
@@ -123,7 +141,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -133,7 +152,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                     chat: dict = None,
                     active: bool = None,
                     continuous: bool = None,
-                    payload: dict = None) -> dict:
+                    payload: dict = None) -> RtmResponse:
         ''' Restarts an archived chat.
 
             Args:
@@ -144,7 +163,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -153,7 +173,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
     def get_chat(self,
                  chat_id: str = None,
                  thread_id: str = None,
-                 payload: dict = None) -> dict:
+                 payload: dict = None) -> RtmResponse:
         ''' It returns a thread that the current Customer has access to in a given chat.
 
             Args:
@@ -163,7 +183,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -173,7 +194,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                    limit: int = None,
                    sort_order: str = None,
                    page_id: str = None,
-                   payload: dict = None) -> dict:
+                   payload: dict = None) -> RtmResponse:
         ''' It returns summaries of the chats a Customer participated in.
 
             Args:
@@ -185,7 +206,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -197,7 +219,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                      limit: int = None,
                      page_id: str = None,
                      min_events_count: int = None,
-                     payload: dict = None) -> dict:
+                     payload: dict = None) -> RtmResponse:
         ''' It returns threads that the current Customer has access to in a given chat.
 
             Args:
@@ -212,13 +234,16 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
         return self.ws.send({'action': 'list_threads', 'payload': payload})
 
-    def deactivate_chat(self, id: str = None, payload: dict = None) -> dict:
+    def deactivate_chat(self,
+                        id: str = None,
+                        payload: dict = None) -> RtmResponse:
         ''' Deactivates a chat by closing the currently open thread.
 
             Args:
@@ -227,7 +252,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -237,7 +263,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                    chat_id: str = None,
                    event: dict = None,
                    attach_to_last_thread: bool = None,
-                   payload: dict = None) -> dict:
+                   payload: dict = None) -> RtmResponse:
         ''' Sends an Event object.
 
             Args:
@@ -248,7 +274,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -257,7 +284,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
     def send_sneak_peek(self,
                         chat_id: str = None,
                         sneak_peek_text: str = None,
-                        payload: dict = None) -> dict:
+                        payload: dict = None) -> RtmResponse:
         ''' Sends a sneak peek to a chat.
 
             Args:
@@ -267,7 +294,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -278,7 +306,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                                    thread_id: str = None,
                                    event_id: str = None,
                                    postback: dict = None,
-                                   payload: dict = None) -> dict:
+                                   payload: dict = None) -> RtmResponse:
         ''' Sends rich message postback.
 
             Args:
@@ -290,7 +318,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -302,7 +331,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
     def accept_greeting(self,
                         greeting_id: int = None,
                         unique_id: str = None,
-                        payload: dict = None) -> dict:
+                        payload: dict = None) -> RtmResponse:
         ''' Marks an incoming greeting as seen.
 
             Args:
@@ -312,7 +341,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -320,7 +350,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
 
     def cancel_greeting(self,
                         unique_id: str = None,
-                        payload: dict = None) -> dict:
+                        payload: dict = None) -> RtmResponse:
         ''' Cancels a greeting.
 
             Args:
@@ -329,13 +359,16 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
         return self.ws.send({'action': 'cancel_greeting', 'payload': payload})
 
-    def get_url_info(self, url: str = None, payload: dict = None) -> dict:
+    def get_url_info(self,
+                     url: str = None,
+                     payload: dict = None) -> RtmResponse:
         ''' It returns the info on a given URL.
 
             Args:
@@ -344,7 +377,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -353,7 +387,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
     def get_form(self,
                  group_id: int = None,
                  type: str = None,
-                 payload: dict = None) -> dict:
+                 payload: dict = None) -> RtmResponse:
         ''' Returns an empty ticket form of a prechat or postchat survey.
 
             Args:
@@ -363,13 +397,14 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
         return self.ws.send({'action': 'get_form', 'payload': payload})
 
-    def get_predicted_agent(self, payload: dict = None) -> dict:
+    def get_predicted_agent(self, payload: dict = None) -> RtmResponse:
         ''' Gets the predicted Agent - the one the Customer will chat with when the chat starts.
 
             Args:
@@ -377,7 +412,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         return self.ws.send({
             'action': 'get_predicted_agent',
@@ -387,7 +423,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
     def mark_events_as_seen(self,
                             chat_id: str = None,
                             seen_up_to: str = None,
-                            payload: dict = None) -> dict:
+                            payload: dict = None) -> RtmResponse:
         ''' Marks events as seen by agent.
 
             Args:
@@ -397,7 +433,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -406,7 +443,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
             'payload': payload
         })
 
-    def get_customer(self, payload: dict = None) -> dict:
+    def get_customer(self, payload: dict = None) -> RtmResponse:
         ''' Returns the info about the customer requesting it.
 
             Args:
@@ -414,7 +451,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         return self.ws.send({
             'action': 'get_customer',
@@ -426,7 +464,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         email: str = None,
                         avatar: str = None,
                         session_fields: list = None,
-                        payload: dict = None) -> dict:
+                        payload: dict = None) -> RtmResponse:
         ''' Updates customer's properties.
 
             Args:
@@ -439,7 +477,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -448,7 +487,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
     def update_customer_page(self,
                              url: str = None,
                              title: str = None,
-                             payload: dict = None) -> dict:
+                             payload: dict = None) -> RtmResponse:
         ''' Updates customer's page.
 
             Args:
@@ -458,7 +497,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -469,7 +509,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
 
     def set_customer_session_fields(self,
                                     session_fields: list = None,
-                                    payload: dict = None) -> dict:
+                                    payload: dict = None) -> RtmResponse:
         ''' Sets customer's session fields.
 
             Args:
@@ -478,7 +518,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -490,7 +531,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
     def delete_chat_properties(self,
                                id: str = None,
                                properties: dict = None,
-                               payload: dict = None) -> dict:
+                               payload: dict = None) -> RtmResponse:
         ''' Deletes chat properties.
 
             Args:
@@ -500,7 +541,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -512,7 +554,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
     def update_chat_properties(self,
                                id: str = None,
                                properties: dict = None,
-                               payload: dict = None) -> dict:
+                               payload: dict = None) -> RtmResponse:
         ''' Updates chat properties.
 
             Args:
@@ -522,7 +564,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -535,7 +578,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                                  chat_id: str = None,
                                  thread_id: str = None,
                                  properties: dict = None,
-                                 payload: dict = None) -> dict:
+                                 payload: dict = None) -> RtmResponse:
         ''' Deletes thread properties.
 
             Args:
@@ -546,7 +589,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -559,7 +603,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                                  chat_id: str = None,
                                  thread_id: str = None,
                                  properties: dict = None,
-                                 payload: dict = None) -> dict:
+                                 payload: dict = None) -> RtmResponse:
         ''' Updates thread properties.
 
             Args:
@@ -570,7 +614,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -584,7 +629,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                                 thread_id: str = None,
                                 event_id: str = None,
                                 properties: dict = None,
-                                payload: dict = None) -> dict:
+                                payload: dict = None) -> RtmResponse:
         ''' Deletes event properties.
 
             Args:
@@ -596,7 +641,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -610,7 +656,7 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                                 thread_id: str = None,
                                 event_id: str = None,
                                 properties: dict = None,
-                                payload: dict = None) -> dict:
+                                payload: dict = None) -> RtmResponse:
         ''' Updates event properties.
 
             Args:
@@ -622,7 +668,8 @@ class CustomerRTMInterface(metaclass=ABCMeta):
                         It overrides all other parameters provided for the method.
 
             Returns:
-                dict: Dictionary with response.
+                RtmResponse: RTM response structure (`request_id`, `action`,
+                             `type`, `success` and `payload` properties)
         '''
         if payload is None:
             payload = prepare_payload(locals())
@@ -660,3 +707,18 @@ class CustomerRTM34(CustomerRTMInterface):
             raise ValueError(
                 'Pipe was not opened. Please check your `organization_id` argument.'
             )
+
+
+class CustomerRTM35(CustomerRTMInterface):
+    ''' Customer RTM version 3.5 class. '''
+    def __init__(self, organization_id: str, version: str,
+                 url: str) -> CustomerRTM35:
+        if isinstance(organization_id, str):
+            self.ws = WebsocketClient(
+                url=
+                f'wss://{url}/v{version}/customer/rtm/ws?organization_id={organization_id}'
+            )
+        else:
+            raise ValueError(
+                'Pipe was not opened. Please check your `organization_id` argument.'
+            )
